from sys import maxsize


def rindex(seq, value, start=0, stop=maxsize, /):
    size = len(seq)

    if start < 0:
        start += size
        if start < 0:
            start = 0

    if size < stop:
        stop = size
    elif stop < 0:
        stop += size
        if stop < 0:
            stop = 0

    i = stop - 1
    while i >= start:
        elem = seq[i]
        if elem is value or elem == value:
            return i
        i -= 1

    raise ValueError("sequence.index(x): x not in sequence")



from operator import indexOf
from itertools import islice


def rindex(seq, value, start=0, stop=None, /):
    size = len(seq)

    if stop is None:
        if start == 0:
            # shortcut common case
            return size - 1 - indexOf(reversed(seq), value)
        stop = 0
    else:
        stop = -stop if stop < 0 else size - stop

    return (
        size
        - 1
        - indexOf(
            islice(reversed(seq), stop, -start if start < 0 else size - start), value
        )
        - stop
    )



__all__ = ("rindex",)

import sys
from functools import wraps
from typing import Sequence, TypeVar


ElemT = TypeVar("ElemT")


def rindex(
    seq: Sequence[ElemT], value: ElemT, start: int = 0, stop: int = None, /
) -> int:
    """Returns last index of value in sequence.

    Raises ValueError if the value is not present.

    Optional `start` and `stop` arguments limit searching to `seq[start:stop]`."""

    ...


update_metadata = wraps(rindex)

if sys.implementation.name == "pypy":
    from .pypy_rindex import rindex
else:
    try:
        from .c_rindex import rindex
    except ImportError:
        from .rindex import rindex

# set annotations and docstring
update_metadata(rindex)
# remove unnecessary functions
del rindex.__wrapped__
del update_metadata



# cython: freethreading_compatible = True

from cython import Py_ssize_t, critical_section

cdef extern from "Python.h":
    Py_ssize_t PY_SSIZE_T_MAX


@critical_section
def rindex(
    seq, value, start: Py_ssize_t = 0, stop: Py_ssize_t = PY_SSIZE_T_MAX, /
) -> Py_ssize_t:
    cdef Py_ssize_t size = len(seq)

    if start < 0:
        start += size
        if start < 0:
            start = 0

    if size < stop:
        stop = size
    elif stop < 0:
        stop += size
        if stop < 0:
            stop = 0

    cdef Py_ssize_t i
    for i in range(stop - 1, start - 1, -1):
        elem = seq[i]
        if elem is value or elem == value:
            return i

    raise ValueError("sequence.index(x): x not in sequence")



import os
import sys

from Cython.Build import cythonize
from setuptools import Extension, setup


enable_exts = os.getenv("ENABLE_RINDEX_EXTENSIONS")
if not enable_exts:
    enable_exts = sys.implementation.name != "pypy"
else:
    try:
        enable_exts = bool(("0", "1").index(enable_exts))
    except ValueError:
        raise ValueError("ENABLE_RINDEX_EXTENSIONS: must be 0, 1 or empty") from None

setup(
    ext_modules=cythonize(
        [
            Extension(
                "rindex.c_rindex",
                sources=["rindex/c_rindex.pyx"],
                language="c",
            )
        ],
        language_level=3,
    )
    if enable_exts
    else None,
)
